#ifndef INCLUDE__LLVMDisassembler_hxx
#define INCLUDE__LLVMDisassembler_hxx

#include <memory>
#include <log4cxx/logger.h>

#include "include_llvm.hxx"

#include "disassembler/Disassembler.hxx"


class LLVMDisassembler
    : public Disassembler {
public:
    LLVMDisassembler(const std::string& filename);
    virtual ~LLVMDisassembler() {};

	void getSymbols();
	uint64_t entryAddress();

    void forEachInstruction(const std::string& name, std::function<void (long, std::string, std::string)> callback) {}

	BasicBlock * generateControlFlowGraph(const std::string& name);
	BasicBlock * generateControlFlowGraph(uint64_t address);

protected:
    bool isFunctionCall(uint64_t address) {return false;}
    bool isJump(uint64_t address) {return false;}

private:
    log4cxx::LoggerPtr logger;

    llvm::Triple triple;
    std::shared_ptr<llvm::object::Binary> binary;


    const llvm::Target * target;
    llvm::object::ObjectFile * o;

    std::unique_ptr<const llvm::MCRegisterInfo> MRI;
    std::unique_ptr<const llvm::MCAsmInfo> AsmInfo;
    std::unique_ptr<llvm::MCModule> Mod;
    std::unique_ptr<llvm::MCInstPrinter> IP;
    std::unique_ptr<llvm::MCDisassembler> DisAsm;
    std::unique_ptr<const llvm::MCObjectFileInfo> MOFI;
    std::unique_ptr<llvm::MCContext> Ctx;
    std::unique_ptr<const llvm::MCInstrAnalysis> MIA;
    std::unique_ptr<const llvm::MCSubtargetInfo> STI;
    std::unique_ptr<const llvm::MCInstrInfo> MII;
	llvm::OwningPtr<llvm::MCRelocationInfo> RelInfo;
	llvm::OwningPtr<llvm::MCSymbolizer> Symzer;

    std::map<std::string, llvm::object::SectionRef> sections;
    std::map<std::string, llvm::object::SymbolRef> symbols;
};

#endif
